unit Sched;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ExtCtrls, ShellAPI;

type
  TfrmMain = class(TForm)
    lstDesc: TListBox;
    lstTime: TListBox;
    lstPath: TListBox;
    lblDesc: TLabel;
    lblTime: TLabel;
    lblPath: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    AddEvent1: TMenuItem;
    RemoveEvent1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Timer: TTimer;
    procedure Exit1Click(Sender: TObject);
    procedure AddEvent1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure RemoveEvent1Click(Sender: TObject);
    procedure lstDescClick(Sender: TObject);
    procedure lstDescDblClick(Sender: TObject);
  private
    { Private declarations }
    strTheTime : String;
    procedure InitializeSchedule;
    procedure ParseFileInput(FileInput : String);
    procedure AddParsedStringToList(ParsedString : String; ParseCount : Integer);
    procedure ExcecuteAnyPresentlyScheduledPrograms;
    procedure SaveConfig;    
  public
    { Public declarations }
  end;

const
    IniFileName = 'Sched.ini';
var
  frmMain: TfrmMain;

implementation

uses Config;

{$R *.DFM}

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TfrmMain.AddEvent1Click(Sender: TObject);
begin
     if frmConfig.ShowModal = IDOK then
     begin
       lstDesc.Items.Add(Trim(frmConfig.editDesc.Text));
       lstTime.Items.Add(Trim(frmConfig.mskEditTime.Text));
       lstPath.Items.Add(Trim(frmConfig.editPath.Text));
       SaveConfig;
     end;
end;

procedure TfrmMain.InitializeSchedule;
var
  IniLineString : String;
  IniFile : TextFile;
begin
  try
    AssignFile(IniFile, IniFileName);
    Reset(IniFile);
    while not Eof(IniFile) do
    begin
      Readln(IniFile, IniLineString);        // Read line out of the file
      IniLineString := Trim(IniLineString);
      ParseFileInput(IniLineString);
    end;
  finally
    CloseFile(IniFile);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
     InitializeSchedule;
end;

procedure TfrmMain.ParseFileInput(FileInput : String);
var
  ParseCount, Size : Integer;
  ParsedString : String;
begin
   ParseCount := 1;
   Size := 0;
   while (Size >= 0) do   // Do as long as a ';' char is found in string
   begin
        ParsedString := FileInput;
        // Below Discards part of string already parsed
        Size := Pos(';', FileInput) - 1;  //Find Position of ';' char in string
        if Size > 0 then
           SetLength(ParsedString, Size);  // This if then prevent setting
                                           // parsed path parmater to 0
        FileInput := Copy(FileInput, Size + 2, Length(FileInput) - 1);
        AddParsedStringToList(ParsedString, ParseCount);
        Inc(ParseCount);
    end;
end;

procedure TfrmMain.AddParsedStringToList(ParsedString : String; ParseCount : Integer);
begin
    case ParseCount of
      1  :   lstDesc.Items.Add(ParsedString);
      2  :   lstTime.Items.Add(ParsedString);
      3  :   lstPath.Items.Add(ParsedString);
    else MessageBox(0, 'Error Occurred While Parseng Your INI File', 'Sched.INI Error', MB_ICONASTERISK);
    end;
end;

procedure TfrmMain.ExcecuteAnyPresentlyScheduledPrograms;
var
  TempString : String;
  TempTimeString : String;
  Index      : Integer;
begin

  for Index := lstTime.Items.Count downto 1 do
  begin
     TempString := lstPath.Items.Strings[Index-1];
     TempString := TempString + #0;  // Add Null Terminator to end Of string
                                     // For use with C style WinAPI Calls
     // if below checks the time in the list box to see if is same as time
     // in the schedule.  The seconds are added to the list box format,
     // listbox format doesn't include seconds.
     TempTimeString := lstTime.Items.Strings[Index-1] + ':00';
     if (TempTimeString = strTheTime) then
     begin
          ShellExecute(0,'Open', @TempString[1],
                  '', '', SW_SHOWNORMAL);  //@NullString[1]
     end;
  end;
end;

procedure TfrmMain.TimerTimer(Sender: TObject);
var
  Present: TDateTime;
 begin
   Present:= Now;
   strTheTime := FormatDateTime('hh:mm:ss', Present);
   ExcecuteAnyPresentlyScheduledPrograms;
  end;

procedure TfrmMain.RemoveEvent1Click(Sender: TObject);
begin
  if (lstTime.ItemIndex > -1) then
     lstTime.Items.Delete(lstTime.ItemIndex);
  if (lstDesc.ItemIndex > -1) then
     lstDesc.Items.Delete(lstDesc.ItemIndex);
  if (lstPath.ItemIndex > -1) then
     lstPath.Items.Delete(lstPath.ItemIndex);
  SaveConfig;
end;

procedure TfrmMain.lstDescClick(Sender: TObject);
begin
  lstTime.ItemIndex := lstDesc.ItemIndex;
  lstPath.ItemIndex := lstDesc.ItemIndex;
end;

procedure TfrmMain.lstDescDblClick(Sender: TObject);
begin
  frmConfig.editDesc.Text := lstDesc.Items.Strings[lstDesc.ItemIndex];
  frmConfig.mskEditTime.Text := lstTime.Items.Strings[lstTime.ItemIndex];
  frmConfig.editPath.Text := lstPath.Items.Strings[lstPath.ItemIndex];
  if frmConfig.ShowModal = IDOK then
  begin
    lstDesc.Items.Strings[lstDesc.ItemIndex] := Trim(frmConfig.editDesc.Text);
    lstTime.Items.Strings[lstTime.ItemIndex] := Trim(frmConfig.mskEditTime.Text);
    lstPath.Items.Strings[lstPath.ItemIndex] := Trim(frmConfig.editPath.Text);
    SaveConfig;
  end;
end;

procedure TfrmMain.SaveConfig;
var
  strScheduleLine : String;
  IniFile : TextFile;
  Index : Integer;
begin
  try
     AssignFile(IniFile, IniFileName);
     Rewrite(IniFile);
     for Index := 0 to lstDesc.Items.Count-1 do
     begin
       strScheduleLine := Trim(lstDesc.Items.Strings[index]) + ';' +
                          Trim(lstTime.Items.Strings[index]) + ';' +
                          Trim(lstPath.Items.Strings[index]);
       writeln(IniFile, strScheduleLine);
     end
  finally
    CloseFile(IniFile);
  end;
end;
end.
